@@ -42,6 +42,8 @@ static GtkToggleButton* mouse_left_handed;
 static GtkRange *kb_delay;
 static GtkRange *kb_interval;
 static GtkToggleButton* kb_beep;
+static GtkButton* kb_layout;
+static GtkLabel* kb_layout_label;
 
 static int accel = 20, old_accel = 20;
 static int threshold = 10, old_threshold = 10;
@@ -55,7 +57,7 @@ static gboolean beep = TRUE, old_beep = TRUE;
 static void on_mouse_accel_changed(GtkRange* range, gpointer user_data)
 {
     accel = (int)gtk_range_get_value(range);
-    XChangePointerControl(GDK_DISPLAY(), True, False,
+    XChangePointerControl(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), True, False,
                              accel, 10, 0);
 }
 
@@ -63,7 +65,7 @@ static void on_mouse_threshold_changed(GtkRange* range, gpointer user_data)
 {
     /* threshold = 110 - sensitivity. The lower the threshold, the higher the sensitivity */
     threshold = 110 - (int)gtk_range_get_value(range);
-    XChangePointerControl(GDK_DISPLAY(), False, True,
+    XChangePointerControl(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), False, True,
                              0, 10, threshold);
 }
 
@@ -71,7 +73,7 @@ static void on_kb_range_changed(GtkRange* range, int* val)
 {
     *val = (int)gtk_range_get_value(range);
     /* apply keyboard values */
-    XkbSetAutoRepeatRate(GDK_DISPLAY(), XkbUseCoreKbd, delay, interval);
+    XkbSetAutoRepeatRate(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), XkbUseCoreKbd, delay, interval);
 }
 
 /* This function is taken from Gnome's control-center 2.6.0.3 (gnome-settings-mouse.c) and was modified*/
@@ -83,11 +85,11 @@ static void set_left_handed_mouse()
     gint idx_1 = 0, idx_3 = 1;
 
     buttons = g_alloca (DEFAULT_PTR_MAP_SIZE);
-    n_buttons = XGetPointerMapping (GDK_DISPLAY(), buttons, DEFAULT_PTR_MAP_SIZE);
+    n_buttons = XGetPointerMapping (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), buttons, DEFAULT_PTR_MAP_SIZE);
     if (n_buttons > DEFAULT_PTR_MAP_SIZE)
     {
         buttons = g_alloca (n_buttons);
-        n_buttons = XGetPointerMapping (GDK_DISPLAY(), buttons, n_buttons);
+        n_buttons = XGetPointerMapping (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), buttons, n_buttons);
     }
 
     for (i = 0; i < n_buttons; i++)
@@ -103,7 +105,7 @@ static void set_left_handed_mouse()
     {
         buttons[idx_1] = ((n_buttons < 3) ? 2 : 3);
         buttons[idx_3] = 1;
-        XSetPointerMapping (GDK_DISPLAY(), buttons, n_buttons);
+        XSetPointerMapping (GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), buttons, n_buttons);
     }
 }
 
@@ -118,7 +120,7 @@ static void on_kb_beep_toggle(GtkToggleButton* btn, gpointer user_data)
     XKeyboardControl values;
     beep = gtk_toggle_button_get_active(btn);
     values.bell_percent = beep ? -1 : 0;
-    XChangeKeyboardControl(GDK_DISPLAY(), KBBellPercent, &values);
+    XChangeKeyboardControl(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), KBBellPercent, &values);
 }
 
 static gboolean on_change_val(GtkRange *range, GtkScrollType scroll,
@@ -128,6 +130,40 @@ static gboolean on_change_val(GtkRange *range, GtkScrollType scroll,
     return FALSE;
 }
 
+static const gchar* detect_keymap_program()
+{
+    const gchar *program = NULL;
+
+    /*find if lxkeymap is in the path*/
+    program = "lxkeymap";
+    if (g_find_program_in_path(program))
+    {
+        return program;
+    }
+    /*TODO
+    Add other keymap executable*/
+    else
+    {
+        return NULL;
+    }
+
+}
+
+static void on_kb_layout_clicked(GtkButton *button,  gpointer   user_data)
+{
+
+    int status;
+    char* output = NULL;
+    const gchar *program = detect_keymap_program();
+
+    if (program)
+    {
+        g_spawn_command_line_sync(program, &output, NULL, &status, NULL );
+    }
+
+}
+
+
 static void set_range_stops(GtkRange* range, int interval )
 {
 /*
@@ -201,6 +237,20 @@ int main(int argc, char** argv)
     kb_delay = (GtkRange*)gtk_builder_get_object(builder,"kb_delay");
     kb_interval = (GtkRange*)gtk_builder_get_object(builder,"kb_interval");
     kb_beep = (GtkToggleButton*)gtk_builder_get_object(builder,"beep");
+    kb_layout = (GtkButton*)gtk_builder_get_object(builder,"keyboard_layout");
+
+    const gchar *program = detect_keymap_program();
+    if (program == NULL)
+    {
+        /* Hide the button if there is no program to set keymap */
+        kb_layout_label = (GtkLabel*)gtk_builder_get_object(builder,"keyboard_layout_label");
+        gtk_widget_set_visible(GTK_WIDGET(kb_layout_label), FALSE);
+        gtk_widget_set_visible(GTK_WIDGET(kb_layout), FALSE);
+    }
+    else
+    {
+            gtk_button_set_label(kb_layout, program);
+    }
 
     g_object_unref( builder );
 
@@ -229,6 +279,7 @@ int main(int argc, char** argv)
     set_range_stops(kb_interval, 10);
     g_signal_connect(kb_interval, "value-changed", G_CALLBACK(on_kb_range_changed), &kb_interval);
     g_signal_connect(kb_beep, "toggled", G_CALLBACK(on_kb_beep_toggle), NULL);
+    g_signal_connect(kb_layout, "clicked", G_CALLBACK(on_kb_layout_clicked), NULL);
 
     if( gtk_dialog_run( (GtkDialog*)dlg ) == GTK_RESPONSE_OK )
     {
@@ -265,14 +316,14 @@ int main(int argc, char** argv)
         delay = old_delay;
         interval = old_interval;
         beep = old_beep;
-        XkbSetAutoRepeatRate(GDK_DISPLAY(), XkbUseCoreKbd, delay, interval);
+        XkbSetAutoRepeatRate(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), XkbUseCoreKbd, delay, interval);
         /* FIXME: beep? */
 
         /* mouse */
         accel = old_accel;
         threshold = old_threshold;
         left_handed = old_left_handed;
-        XChangePointerControl(GDK_DISPLAY(), True, True,
+        XChangePointerControl(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), True, True,
                                  accel, 10, threshold);
         set_left_handed_mouse();
     }
